/*
 *  Copyright (C) 2002 Marco Pesenti Gritti
 *  Copyright (C) 2003 Philip Langdale (Egg port)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ppview-toolbar.h"
#include "galeon-window.h"
#include "gul-string.h"
#include "gul-gui.h"

#include <string.h>
#include <libgnome/gnome-i18n.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkmenu.h>
#include "egg-action.h"
#include "egg-action-group.h"
#include "egg-dock.h"

#define ACTION_ITEM(name) egg_action_group_get_action(t->priv->action_group, name)

#define PPV_GOTO_FIRST_ACTION "PPVGotoFirstAction"
#define PPV_GOTO_LAST_ACTION "PPVGotoLastAction"
#define PPV_GO_BACK_ACTION "PPVGoBackAction"
#define PPV_GO_FORWARD_ACTION "PPVGoForwardAction"

static void ppview_toolbar_class_init(PPViewToolbarClass *klass);
static void ppview_toolbar_init(PPViewToolbar *t);
static void ppview_toolbar_finalize(GObject *object);
static void ppview_toolbar_set_window(PPViewToolbar *t, GaleonWindow *window);
static void
ppview_toolbar_set_property(GObject *object,
                            guint prop_id,
                            const GValue *value,
                            GParamSpec *pspec);
static void
ppview_toolbar_get_property(GObject *object,
                            guint prop_id,
                            GValue *value,
                            GParamSpec *pspec);

enum
{
	PROP_0,
	PROP_GALEON_WINDOW
};

static GObjectClass *parent_class = NULL;

struct PPViewToolbarPrivate
{
	GaleonWindow *window;
	EggDockItem *dock_item;
	EggActionGroup *action_group;
	gboolean visibility;
	EmbedChromeMask old_chrome;
	int current_page;
};

static void 
toolbar_cmd_ppv_goto_first(EggAction *action, 
	 		   PPViewToolbar *t);

static void 
toolbar_cmd_ppv_goto_last(EggAction *action, 
			  PPViewToolbar *t);

static void 
toolbar_cmd_ppv_go_back(EggAction *action, 
			PPViewToolbar *t);

static void 
toolbar_cmd_ppv_go_forward(EggAction *action,
			   PPViewToolbar *t);

static void 
toolbar_cmd_ppv_close(EggAction *action,
		      PPViewToolbar *t);

static EggActionGroupEntry ppv_entries[] = {
	{ "PPVGotoFirstAction", N_("First"), GTK_STOCK_GOTO_FIRST, NULL,
	  N_("Go to the first page"), G_CALLBACK(toolbar_cmd_ppv_goto_first), NULL },
	{ "PPVGoBackAction", N_("Previous"), GTK_STOCK_GO_BACK, NULL,
	  N_("Go to the previous page"), G_CALLBACK(toolbar_cmd_ppv_go_back), NULL },
	{ "PPVGoForwardAction", N_("Next"), GTK_STOCK_GO_FORWARD, NULL,
	  N_("Go to the next page"), G_CALLBACK(toolbar_cmd_ppv_go_forward), NULL },
	{ "PPVGotoLastAction", N_("Last"), GTK_STOCK_GOTO_LAST, NULL,
	  N_("Go to the last page"), G_CALLBACK(toolbar_cmd_ppv_goto_last), NULL },
	{ "PPVCloseAction", N_("Close"), GTK_STOCK_CLOSE, NULL,
	  N_("Leave print preview mode"), G_CALLBACK(toolbar_cmd_ppv_close),NULL }
};
static guint n_ppv_entries = G_N_ELEMENTS(ppv_entries);

GType 
ppview_toolbar_get_type (void)
{
        static GType ppview_toolbar_type = 0;

        if (ppview_toolbar_type == 0)
        {
                static const GTypeInfo our_info =
                {
                        sizeof (PPViewToolbarClass),
                        NULL, /* base_init */
                        NULL, /* base_finalize */
                        (GClassInitFunc) ppview_toolbar_class_init,
                        NULL,
                        NULL, /* class_data */
                        sizeof (PPViewToolbar),
                        0, /* n_preallocs */
                        (GInstanceInitFunc) ppview_toolbar_init
                };

                ppview_toolbar_type = g_type_register_static (G_TYPE_OBJECT,
						       "PPViewToolbar",
						       &our_info, 0);
        }

        return ppview_toolbar_type;

}

static void
ppview_toolbar_class_init (PPViewToolbarClass *klass)
{
        GObjectClass *object_class = G_OBJECT_CLASS (klass);

        parent_class = g_type_class_peek_parent (klass);

        object_class->finalize = ppview_toolbar_finalize;

	object_class->set_property = ppview_toolbar_set_property;
	object_class->get_property = ppview_toolbar_get_property;

	g_object_class_install_property (object_class,
                                         PROP_GALEON_WINDOW,
                                         g_param_spec_object ("GaleonWindow",
                                                              "GaleonWindow",
                                                              "Parent window",
                                                              GALEON_WINDOW_TYPE,
                                                              G_PARAM_READWRITE));
}

static void
ppview_toolbar_set_property (GObject *object,
                      guint prop_id,
                      const GValue *value,
                      GParamSpec *pspec)
{
        PPViewToolbar *t = PPVIEW_TOOLBAR (object);

        switch (prop_id)
        {
                case PROP_GALEON_WINDOW:
                        ppview_toolbar_set_window (t, g_value_get_object (value));
                        break;
        }
}

static void
ppview_toolbar_get_property (GObject *object,
                      guint prop_id,
                      GValue *value,
                      GParamSpec *pspec)
{
        PPViewToolbar *t = PPVIEW_TOOLBAR (object);

        switch (prop_id)
        {
                case PROP_GALEON_WINDOW:
                        g_value_set_object (value, t->priv->window);
                        break;
        }
}
	
static void
ppview_toolbar_set_window (PPViewToolbar *t, GaleonWindow *window)
{
	unsigned int i;

	g_return_if_fail (t->priv->window == NULL);
	
	t->priv->window = window;

	t->priv->dock_item =
		egg_dock_get_item_by_name(EGG_DOCK(window->egg_dock),
					  "PPVToolbar");

	t->priv->action_group = egg_action_group_new("PPVActions");
	for (i=0 ; i < n_ppv_entries ; i++)
	{
		ppv_entries[i].user_data = t;
	}

	egg_action_group_add_actions(t->priv->action_group,
				     ppv_entries,
				     n_ppv_entries);

	egg_menu_merge_insert_action_group(window->merge,
					   t->priv->action_group, 0);
}

static void
ppview_toolbar_init (PPViewToolbar *t)
{
        t->priv = g_new0 (PPViewToolbarPrivate, 1);

	t->priv->window = NULL;
	t->priv->dock_item = NULL;
	t->priv->visibility = TRUE;
	t->priv->action_group = NULL;
}

static void
ppview_toolbar_finalize (GObject *object)
{
	PPViewToolbar *t;

        g_return_if_fail (object != NULL);
        g_return_if_fail (IS_PPVIEW_TOOLBAR (object));

	t = PPVIEW_TOOLBAR (object);

        g_return_if_fail (t->priv != NULL);

	g_object_unref(t->priv->action_group);

        g_free (t->priv);

        G_OBJECT_CLASS (parent_class)->finalize (object);
}

PPViewToolbar *
ppview_toolbar_new (GaleonWindow *window)
{
	PPViewToolbar *t;

	t = PPVIEW_TOOLBAR (g_object_new (PPVIEW_TOOLBAR_TYPE, 
				   	  "GaleonWindow", window,
				   	  NULL));

	g_return_val_if_fail (t->priv != NULL, NULL);

	return t;
}

void
ppview_toolbar_set_old_chrome (PPViewToolbar *t, 
			       EmbedChromeMask chrome)
{
	t->priv->old_chrome = chrome;
}

static void
toolbar_update_sensitivity (PPViewToolbar *t)
{
	int pages, c_page;
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	galeon_embed_print_preview_num_pages (embed, &pages);
	c_page = t->priv->current_page;

	g_object_set(G_OBJECT(ACTION_ITEM(PPV_GO_BACK_ACTION)),
		     "sensitive", c_page > 1, NULL);
	g_object_set(G_OBJECT(ACTION_ITEM(PPV_GOTO_FIRST_ACTION)),
		     "sensitive", c_page > 1, NULL);
	g_object_set(G_OBJECT(ACTION_ITEM(PPV_GO_FORWARD_ACTION)),
		     "sensitive", c_page < pages, NULL);
	g_object_set(G_OBJECT(ACTION_ITEM(PPV_GOTO_LAST_ACTION)),
		     "sensitive", c_page < pages, NULL);
}

void
ppview_toolbar_set_visibility (PPViewToolbar *t, gboolean visibility)
{
	if (visibility == t->priv->visibility) return;
	
	t->priv->visibility = visibility;

	if (visibility)
	{
		t->priv->current_page = 1;
		toolbar_update_sensitivity (t);

		gtk_widget_show(GTK_WIDGET(t->priv->dock_item));
	}
	else
	{
		gtk_widget_hide(GTK_WIDGET(t->priv->dock_item));
	}

}

static void 
toolbar_cmd_ppv_goto_first(EggAction *action, 
			   PPViewToolbar *t)
{
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	galeon_embed_print_preview_navigate (embed, PRINTPREVIEW_HOME, 0);

	t->priv->current_page = 1;

	toolbar_update_sensitivity (t);
}

static void 
toolbar_cmd_ppv_goto_last(EggAction *action, 
			  PPViewToolbar *t)
{
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	galeon_embed_print_preview_navigate (embed, 
					     PRINTPREVIEW_END, 
					     0);

	galeon_embed_print_preview_num_pages (embed, 
					      &t->priv->current_page);

	toolbar_update_sensitivity (t);
}

static void 
toolbar_cmd_ppv_go_back(EggAction *action, 
			PPViewToolbar *t)
{
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);

	t->priv->current_page --;

	galeon_embed_print_preview_navigate (embed, 
					     PRINTPREVIEW_GOTO_PAGENUM,
					     t->priv->current_page);
	toolbar_update_sensitivity (t);
}

static void 
toolbar_cmd_ppv_go_forward(EggAction *action, 
			   PPViewToolbar *t)
{
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (embed != NULL);
	
	t->priv->current_page ++;

	galeon_embed_print_preview_navigate (embed, 
					     PRINTPREVIEW_GOTO_PAGENUM,
					     t->priv->current_page);
	toolbar_update_sensitivity (t);
}

static void 
toolbar_cmd_ppv_close(EggAction *action, 
		      PPViewToolbar *t)
{
	GaleonWindow *window = t->priv->window;
	GaleonEmbed *embed;

	g_return_if_fail (IS_GALEON_WINDOW (window));
	
	embed = galeon_window_get_active_embed (window);
	g_return_if_fail (IS_GALEON_EMBED (embed));

	galeon_window_set_chrome (window, t->priv->old_chrome);
	
	galeon_embed_print_preview_close (embed);

	toolbar_update_sensitivity (t);
}

